package uprrp.tania.models;

import android.util.Log;

public class UserStatusModel {

    private static final String TAG = "UserStatusModel";
    // NOTE: these attributes must have the same name
    // as in the JSON and need not be manually initialized
    private String institucion;
    private String experiencia;
    private int surveys_completed;
    private int surveys_sent;
    private String time_to_next;

    public boolean isEnrolled() {
        return true;
    }

    public boolean surveyAvailable() {
        // WARNING: Condition might change in the future. Don't change if statement for clarity.
        if(this.time_to_next.equals("::") || this.time_to_next.equals("0:0:0")) {
            return true;
        } else { // TODO: add comparison to null, which signifies experience has ran out of moments
            return false;
        }
    }

    public boolean ranOutOfMoments() {
        return this.time_to_next == null; // TODO: confirm this is going to be used
    }

    public String getTimeToNext() {
        if(this.ranOutOfMoments()) {
            return "No surveys for now. Come back later!";
        } else if(this.surveyAvailable()) {
            return "Survey available!";
        } else {
            return "Next survey in " + this.formatTimeToNext() + ".";
        }
    }

    private String formatTimeToNext() {

        // Format is DD:HH:MM
        String[] parts = this.time_to_next.split(":");
        int days = Integer.parseInt(parts[0]);
        int hours = Integer.parseInt(parts[1]);
        int minutes = Integer.parseInt(parts[2]);
        String formattedTimeToNext = "";

        if(days != 0) {
            formattedTimeToNext += days + " days";
            if(hours != 0) {
                formattedTimeToNext += ", " + hours + " hours";
            } else {
                formattedTimeToNext += ", " + minutes + " minutes";
            }
        } else {
            if(hours != 0) {
                formattedTimeToNext += hours + " hours";
                formattedTimeToNext += ", " + minutes + " minutes";
            } else {
                if(minutes != 0) {
                    formattedTimeToNext += minutes + " minutes";
                } else {
                    Log.wtf(TAG, "Encountered a 0:0:0 in UserStatusModel.formatTimeToNext()!\nShould've been handled by UserStatusModel.surveyAvailable()");
                }
            }
        }

        return formattedTimeToNext;

    }

    public String toString() {

        // If user is not enrolled in an experience,
        // return early with this string
        if(!this.isEnrolled()) {
            return "You are not enrolled in any experience.\nIf you think this is a mistake, contact your supervisors.";
        }

        // Else, return proper string
        String status = "You're enrolled in " + this.experiencia + ", by " + this.institucion + ".";
        if(this.surveys_completed == this.surveys_sent) {
            status += " You've answered all surveys. Come back later in case a new one pops up!";
        } else {
            status += " So far, you have completed " + this.surveys_completed + " out of " + this.surveys_sent + " delivered surveys.";
        }
        return status;

    }

}
